# Generated by Django 2.1.2 on 2018-10-31 21:21

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('spider_verifier', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='dataverificationtag',
            options={'permissions': [('can_verify', 'Can verify Data Tag?')]},
        ),
        migrations.AlterField(
            model_name='dataverificationtag',
            name='note',
            field=models.TextField(blank=True, default=''),
        ),
    ]
