# Generated by Django 2.1.5 on 2019-01-09 15:05

from django.db import migrations, models
import spkcspider.apps.spider_accounts.models


class Migration(migrations.Migration):

    dependencies = [
        ('spider_accounts', '0003_spideruser_quota'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='spideruser',
            name='quota',
        ),
        migrations.AddField(
            model_name='spideruser',
            name='quota_local',
            field=models.PositiveIntegerField(blank=True, default=spkcspider.apps.spider_accounts.models.default_quota_spider_user_local, help_text='Quota in Bytes, null for no limit', null=True),
        ),
        migrations.AddField(
            model_name='spideruser',
            name='quota_remote',
            field=models.PositiveIntegerField(blank=True, default=spkcspider.apps.spider_accounts.models.default_quota_spider_user_remote, help_text='Quota in Bytes, null for no limit', null=True),
        ),
    ]
