# Generated by Django 2.1.5 on 2019-01-15 01:47

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('spider_base', '0008_authtoken_referrer'),
    ]

    operations = [
        migrations.AddField(
            model_name='authtoken',
            name='persist',
            field=models.BooleanField(blank=True, db_index=True, default=False),
        ),
    ]
