# Generated by Django 2.1.5 on 2019-01-17 12:53

from django.db import migrations, models
import jsonfield.fields


class Migration(migrations.Migration):

    dependencies = [
        ('spider_tags', '0003_auto_20180815_1205'),
    ]

    operations = [
        migrations.AddField(
            model_name='taglayout',
            name='unique',
            field=models.BooleanField(blank=True, default=False),
        ),
        migrations.AlterField(
            model_name='taglayout',
            name='layout',
            field=jsonfield.fields.JSONField(default=list, help_text='Field list in JSON format'),
        ),
    ]
