# Generated by Django 2.1.5 on 2019-01-09 21:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('spider_base', '0007_linkcontent_push'),
    ]

    operations = [
        migrations.AddField(
            model_name='authtoken',
            name='referrer',
            field=models.URLField(blank=True, max_length=400, null=True),
        ),
    ]
