# Generated by Django 2.1 on 2018-08-21 00:21

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('spider_filets', '0003_auto_20180820_1409'),
    ]

    operations = [
        migrations.AlterField(
            model_name='textfilet',
            name='editable_from',
            field=models.ManyToManyField(blank=True, help_text='Allow editing from selected components.', related_name='_textfilet_editable_from_+', to='spider_base.UserComponent'),
        ),
    ]
