# Generated by Django 2.1 on 2018-08-10 00:44

from django.db import migrations, models
import spkcspider.apps.spider_filets.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='FileFilet',
            fields=[
                ('id', models.BigAutoField(editable=False, primary_key=True, serialize=False)),
                ('add', models.DateTimeField(auto_now_add=True)),
                ('modified', models.DateTimeField(auto_now=True)),
                ('name', models.CharField(max_length=255)),
                ('file', models.FileField(upload_to=spkcspider.apps.spider_filets.models.get_file_path)),
            ],
            options={
                'abstract': False,
                'default_permissions': [],
            },
        ),
        migrations.CreateModel(
            name='TextFilet',
            fields=[
                ('id', models.BigAutoField(editable=False, primary_key=True, serialize=False)),
                ('name', models.CharField(max_length=255)),
                ('text', models.TextField(default='')),
            ],
            options={
                'abstract': False,
                'default_permissions': [],
            },
        ),
    ]
