# Generated by Django 2.1.5 on 2019-01-20 12:52

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('spider_base', '0012_auto_20190118_1017'),
    ]

    operations = [
        migrations.AddField(
            model_name='assignedcontent',
            name='persist_token',
            field=models.ForeignKey(blank=True, limit_choices_to={'persist__gte': 0}, null=True, on_delete=django.db.models.deletion.CASCADE, to='spider_base.AuthToken'),
        ),
    ]
