{# just use edit_form, real verifier have to replace this or to depend on spider #}
{% extends "spider_base/base.html" %}
{% load i18n spider_rdf %}

{% block main_attributes %}{{block.super}} prefix="spkc: {% spkc_namespace %} xsd: http://www.w3.org/2001/XMLSchema#" resource="{{ request.path }}"{% endblock %}

{% block content %}
<form action="{{ request.get_full_path }}" enctype="multipart/form-data" method="post">
  {% include "spider_base/edit_form.html" with inner_form=True legend='Send spider or content to verify' %}
  <button class="w3-button w3-grey">
    {% trans 'Submit' %}
  </button>
</form>
{% endblock content %}
