{% extends "registration/base.html" %}
{% load i18n %}
{% block extrahead %}
{{block.super}}
<meta http-equiv="refresh" content="1; URL={% url 'auth:login' %}">
{% endblock %}


{% block content %}
  <p>{% trans "Logout successful." %}</p>
  <a href="{% url 'auth:login' %}">{% trans 'Click if not redirected' %}</a>
{% endblock %}
{# form errors should not appear here #}
{% block errors %}{% endblock errors%}
