# Generated by Django 2.1.5 on 2019-01-09 15:05

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('spider_base', '0004_auto_20190103_0156'),
    ]

    operations = [
        migrations.RenameField(
            model_name='userinfo',
            old_name='used_space',
            new_name='used_space_local',
        ),
        migrations.AddField(
            model_name='userinfo',
            name='used_space_remote',
            field=models.BigIntegerField(default=0, editable=False),
        ),
    ]
