# Generated by Django 2.1.5 on 2019-01-15 21:49

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('spider_base', '0009_authtoken_persist'),
    ]

    operations = [
        migrations.AddField(
            model_name='usercomponent',
            name='can_auth',
            field=models.BooleanField(default=False, editable=False),
        ),
    ]
