# Generated by Django 2.1.2 on 2018-10-31 15:06

from django.db import migrations, models
import spkcspider.apps.verifier.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='DataVerificationTag',
            fields=[
                ('id', models.BigAutoField(editable=False, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('modified', models.DateTimeField(auto_now=True)),
                ('hash', models.SlugField(max_length=512, unique=True)),
                ('dvfile', models.FileField(blank=True, help_text='File with data to verify', null=True, upload_to=spkcspider.apps.verifier.models.dv_path)),
                ('data_type', models.CharField(default='layout', max_length=20)),
                ('checked', models.DateTimeField(blank=True, null=True)),
                ('verification_state', models.CharField(choices=[('pending', 'pending'), ('verified', 'verified'), ('rejected', 'rejected')], default='pending', max_length=10)),
                ('note', models.TextField(default='')),
            ],
        ),
    ]
