# Generated by Django 2.1.4 on 2019-01-02 22:57

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('spider_base', '0002_20181231_squashed'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='usercomponent',
            name='avail_features',
        ),
    ]
