# Generated by Django 2.1.2 on 2018-10-07 18:31

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('spider_filets', '0005_auto_20180919_1329'),
    ]

    operations = [
        migrations.AddField(
            model_name='textfilet',
            name='preview_words',
            field=models.PositiveIntegerField(default=10, help_text='How many words should be used for preview?'),
        ),
    ]
