# Generated by Django 2.1.5 on 2019-01-28 22:51

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('spider_base', '0005_auto_20190128_2230'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='assignedcontent',
            name='nonce',
        ),
        migrations.RemoveField(
            model_name='usercomponent',
            name='nonce',
        ),
    ]
