{% load spider_rdf %}
<div class="w3-padding">
  <fieldset style="width:100%">
    {% for rdftype in add_spkc_types %}
      <data property="spkc:type" hidden="hidden">{{rdftype}}</data>
    {% endfor %}
    <legend><h2>{{legend|safe}}</h2></legend>
    {% include "spider_base/partials/base_view_form.html" with form=form %}
  </fieldset>
</div>
