# Generated by Django 2.1.7 on 2019-03-17 14:05

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('spider_base', '0008_auto_20190225_2034'),
    ]

    operations = [
        migrations.AddField(
            model_name='assignedcontent',
            name='allow_domain_mode',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='assignedcontent',
            name='attached_to_content',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='attached_contents', to='spider_base.AssignedContent'),
        ),
        migrations.AddField(
            model_name='assignedcontent',
            name='features',
            field=models.ManyToManyField(blank=True, limit_choices_to=models.Q(ctype__contains='g'), related_name='feature_for_contents', to='spider_base.ContentVariant'),
        ),
        migrations.AlterField(
            model_name='usercomponent',
            name='features',
            field=models.ManyToManyField(blank=True, limit_choices_to=models.Q(ctype__contains='f'), related_name='feature_for_components', to='spider_base.ContentVariant'),
        ),
    ]
