# Generated by Django 2.1.4 on 2019-01-03 01:56

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('spider_base', '0003_remove_usercomponent_avail_features'),
    ]

    operations = [
        migrations.AlterField(
            model_name='assignedcontent',
            name='ctype',
            field=models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.SET_NULL, to='spider_base.ContentVariant'),
        ),
    ]
