# Generated by Django 2.1.5 on 2019-01-09 19:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('spider_base', '0005_auto_20190109_1505'),
    ]

    operations = [
        migrations.AddField(
            model_name='assignedcontent',
            name='priority',
            field=models.SmallIntegerField(blank=True, default=0),
        ),
    ]
