{% extends "registration/base.html" %}
{% load i18n static %}
{% block main_classes %}{{block.super}} w3-animate-bottom{% endblock %}
{% block content %}
  <form action="{{ app_path }}" method="post">
    {% csrf_token %}
    {# don't annotate (correctly) for making it harder for bots #}
    <fieldset>
      <legend><h1>{% trans 'Sign up' %}</h1></legend>
      {% include "spider_base/partials/base_form.html" with form=form %}
    </fieldset>
    <button class="w3-button w3-grey" type="submit">{% trans 'Sign up' %}</button><br/>
    <a href="{% url 'auth:login' %}">{% trans 'Already an account? Login?' %}</a>
  </form>


{% endblock %}
