{% extends "registration/base.html" %}
{% load i18n static %}
{% block title %}{{ title }}{% endblock %}

{% block content %}
  <form method="post">
    {% csrf_token %}
    <fieldset>
      <legend><h1>{% trans 'Change Password' %}</h1></legend>
      <p>{% trans "Please enter your old password, for security's sake, and then enter your new password twice so we can verify you typed it in correctly." %}</p>
      {% include "spider_base/partials/base_form.html" with form=form %}
    </fieldset>
    <button class="w3-button w3-grey" type="submit">{% trans 'Change my password' %}</button>
  </form>
{% endblock %}
