<div class="w3-padding">
  <h1>Key & hash</h1>
  <table>
    <tr>
      <td>Hash ({{algo}})</td>
      <td style="word-break: break-all">{{hash}}</td>
    </tr>
    <tr style="margin-top:5px">
      <td>key</td>
      <td style="word-break: break-all">{{object.key}}</td>
    </tr>
  </table>
</div>
