# Generated by Django 2.1.2 on 2018-11-02 18:26

from django.db import migrations
import jsonfield.fields


class Migration(migrations.Migration):

    dependencies = [
        ('spider_verifier', '0002_auto_20181031_2121'),
    ]

    operations = [
        migrations.AddField(
            model_name='dataverificationtag',
            name='linked_hashes',
            field=jsonfield.fields.JSONField(blank=True, default=dict),
        ),
    ]
